package com.muyu.task.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.domain.DataValue;
import com.muyu.common.domian.NodeJoint;
import com.muyu.common.domian.TaskInfo;
import com.muyu.common.domian.TaskInput;
import com.muyu.common.domian.TaskOutput;
import com.muyu.common.domian.enums.Weight;
import com.muyu.common.domian.req.TaskInfoListReq;
import com.muyu.common.domian.resp.TaskInfoResp;
import com.muyu.remote.feign.DatasourceFeign;
import com.muyu.task.server.mapper.TaskInfoMapper;
import com.muyu.task.server.service.NodeJointService;
import com.muyu.task.server.service.TaskInfoService;
import com.muyu.task.server.service.TaskInputService;
import com.muyu.task.server.service.TaskOutputService;
import lombok.extern.log4j.Log4j2;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import static com.muyu.task.server.thread.OptimizedPrioritizedThreadPool.*;

/**
 * @author Administrator
 */
@Log4j2
@Service
public class TaskInfoServiceImpl extends ServiceImpl<TaskInfoMapper, TaskInfo> implements TaskInfoService {

    @Resource
    private TaskInfoMapper taskInfoMapper;
    @Autowired
    private TaskInputService taskInputService;

    @Autowired
    private NodeJointService nodeJointService;

    @Autowired
    private TaskOutputService taskOutputService;


    @Autowired
    private DatasourceFeign datasourceFeign;

    @Override
    public Page<TaskInfoResp> selectList(TaskInfoListReq req) {
        LambdaQueryWrapper<TaskInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(req.getName()), TaskInfo::getName, req.getName());
        queryWrapper.eq(req.getWeight() != null, TaskInfo::getWeight, req.getWeight());
        queryWrapper.eq(StringUtils.isNotEmpty(req.getProcessStatus()), TaskInfo::getProcessStatus, req.getProcessStatus());
        Page<TaskInfo> page = new Page<>(req.getPageNum(), req.getPageSize());
        Page<TaskInfo> taskInfoPage = taskInfoMapper.selectPage(page, queryWrapper);
        long pages = taskInfoPage.getPages();
        long size = taskInfoPage.getSize();
        long current = taskInfoPage.getCurrent();
        List<TaskInfoResp> list = taskInfoPage.getRecords().stream().map(taskInfo -> TaskInfo.build(taskInfo)).toList();
        Page<TaskInfoResp> respPage = new Page<>();
        respPage.setRecords(list);
        respPage.setCurrent(current);
        respPage.setSize(size);
        respPage.setTotal(this.count(queryWrapper));
        respPage.setPages(pages);
        return respPage;
    }
//
//    @Override
//    public String findByFieName(Long taskId) {
//        QueryWrapper<TaskInput> wrapper = new QueryWrapper<>();
//        wrapper.eq("task_id", taskId);
//        HashSet<Long> longs = new HashSet<>();
//        List<TaskInput> list = taskInputService.list(wrapper);
//        if (CollectionUtils.isEmpty(list)) {
//            return "没有选择字段";
//        }
//        String fieName = "";
//        HashMap<String, Long> hashMap = new HashMap<>();
//        for (int i = 0; i < list.size(); i++) {
//            fieName += "," + list.get(i).getTableAsFieId();
//            Long databaseId = list.get(i).getDatabaseId();
//            longs.add(databaseId);
//            String tableFieId = list.get(i).getTableFieId();
//            Long fieIdAsEngineId = list.get(i).getFieIdAsEngineId();
//            hashMap.put(tableFieId,fieIdAsEngineId);
//        }
//        if (longs.size() > 1) {
//            return "你选择的不是同一个数据库";
//        }
//
//        QueryWrapper<NodeJoint> jointQueryWrapper = new QueryWrapper<>();
//        jointQueryWrapper.eq("task_id", taskId);
//        List<NodeJoint> jointList = nodeJointService.list(jointQueryWrapper);
//        String joint = "";
//        if (!CollectionUtils.isEmpty(jointList)) {
//            for (NodeJoint nodeJoint : jointList) {
//                String oneNodeId = nodeJoint.getOneNodeId();
//                TaskInput taskInput = taskInputService.selectByNodeId(oneNodeId);
//                String tableName = taskInput.getTableName();
//                String tableAsName = taskInput.getTableAsName();
//
//                String twoNodeId = nodeJoint.getTwoNodeId();
//                TaskInput taskInputTwo = taskInputService.selectByNodeId(twoNodeId);
//                String tableNameTwo = taskInputTwo.getTableName();
//                String tableAsNameTwo = taskInputTwo.getTableAsName();
//                joint += " " + tableName + " " + " " + tableAsName + " " + nodeJoint.getJoint()
//                        + " " + tableNameTwo + " " + " " + tableAsNameTwo + " on " + nodeJoint.getOneFie()
//                        + " = " + nodeJoint.getTwoFie();
//
//            }
//        }else {
//            TaskInput taskInput = taskInputService.selectByTaskId(taskId);
//            String tableName = taskInput.getTableName();
//            String tableAsName = taskInput.getTableAsName();
//            joint=" "+ tableName+" "+tableAsName;
//        }
//        Long basicId = taskInputService.selectByBasicId(taskId);
//        fieName = fieName.substring(1);
//        String sql = " SELECT " + fieName + " FROM "+ joint;
//        System.out.println(sql);
//        Result<List<List<DataValue>>> tableValueResult = datasourceFeign.findTableValue(basicId, sql);
//        QueryWrapper<TaskOutput> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("task_id", taskId);
//        List<TaskOutput> list1 = taskOutputService.list(queryWrapper);
//        HashMap<String, String> map = new HashMap<>();
//        Long tableId=null;
//        Long newBasicId=null;
//        for (TaskOutput taskOutput : list1) {
//            tableId=taskOutput.getTableId();
//            newBasicId=taskOutput.getBasicId();
//            String lastFieName = taskOutput.getLastFieName();
//            String newFieName = taskOutput.getNewFieName();
//            map.put(lastFieName, newFieName);
//        }
//        List<List<DataValue>> tableValue = tableValueResult.getData();
//        for (List<DataValue> dataValues : tableValue) {
//            for (DataValue dataValue : dataValues) {
//                String key = dataValue.getKey();
//                String newKey = map.get(key);
//                dataValue.setKey(newKey);
//            }
//        }
//        return "执行成功";
//    }

    @Override
    public String findByFieName2(Long taskId) {
        TaskInfo taskInfo = taskInfoMapper.selectById(taskId);
        String weight = taskInfo.getWeight();
        QueryWrapper<TaskInput> wrapper = new QueryWrapper<>();
        wrapper.eq("task_id", taskId);
        HashSet<Long> longs = new HashSet<>();
        List<TaskInput> list = taskInputService.list(wrapper);
        if (CollectionUtils.isEmpty(list)) {
            return "没有选择字段";
        }
        HashMap<String, String> tableNameMap = new HashMap<>();
        String fieName = "";
        HashMap<String, String> hashMap = new HashMap<>();
        for (int i = 0; i < list.size(); i++) {
            Long databaseId = list.get(i).getDatabaseId();
            longs.add(databaseId);
            String[] tableFieId = list.get(i).getTableFieId().split(",");
            String[] tableAsFieId = list.get(i).getTableAsFieId().split(",");
            String[] fieIdAsEngineId = list.get(i).getFieIdAsEngineId().split(",");
            for (int j = 0; j < tableFieId.length; j++) {
                hashMap.put(tableFieId[j], fieIdAsEngineId[j]);
                tableNameMap.put(tableFieId[j], tableAsFieId[j]);
            }

        }
        if (longs.size() > 1) {
            return "你选择的不是同一个数据库";
        }

        QueryWrapper<NodeJoint> jointQueryWrapper = new QueryWrapper<>();
        jointQueryWrapper.eq("task_id", taskId);
        List<NodeJoint> jointList = nodeJointService.list(jointQueryWrapper);
        String joint = "";
        if (!CollectionUtils.isEmpty(jointList)) {
            for (NodeJoint nodeJoint : jointList) {
                String oneNodeId = nodeJoint.getOneNodeId();
                QueryWrapper<TaskInput> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("node_id", oneNodeId);
                TaskInput taskInput = taskInputService.getOne(queryWrapper);
                String tableName = taskInput.getTableName();
                String tableAsName = taskInput.getTableAsName();

                String twoNodeId = nodeJoint.getTwoNodeId();
                QueryWrapper<TaskInput> queryWrapperTwo = new QueryWrapper<>();
                queryWrapperTwo.eq("node_id", twoNodeId);
                TaskInput taskInputTwo = taskInputService.getOne(queryWrapperTwo);
                String tableNameTwo = taskInputTwo.getTableName();
                String tableAsNameTwo = taskInputTwo.getTableAsName();
                joint += " " + tableName + " " + " " + tableAsName + " " + nodeJoint.getJoint()
                        + " " + tableNameTwo + " " + " " + tableAsNameTwo + " on " + nodeJoint.getOneFie()
                        + " = " + nodeJoint.getTwoFie();

            }
        } else {
            TaskInput taskInput = taskInputService.getOne(wrapper);
            String tableName = taskInput.getTableName();
            String tableAsName = taskInput.getTableAsName();
            joint = " " + tableName + " " + tableAsName;
        }
        Long basicId = taskInputService.selectByBasicId(taskId);


        QueryWrapper<TaskOutput> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("task_id", taskId);
        TaskOutput one = taskOutputService.getOne(queryWrapper);
        Long tableId = one.getTableId();
        Long newBasicId = one.getBasicId();
        HashMap<String, String> map = new HashMap<>();
        String[] newFieName = one.getNewFieName().split(",");
        String[] lastFieName = one.getLastFieName().split(",");
        for (int i = 0; i < newFieName.length; i++) {
            map.put(lastFieName[i], newFieName[i]);
            fieName += "," + tableNameMap.get(lastFieName[i]);
        }

        fieName = fieName.substring(1);
        String sql = " SELECT count(1) FROM " + joint;
        Result<Long> countResult = datasourceFeign.findCount(basicId, sql);
        Long data = countResult.getData();
        System.out.println("======="+data+"=======");
        String finalFieName = fieName;
        String finalJoint = joint;
        System.out.println(weight);
        System.out.println(Weight.centre.getValue());
        long count = data/10000==0?1:data/10000;
        if (Weight.high.getValue().equals(weight)){

            for (long i = 1; i <= count; i++) {
               long pageNum = (i - 1) * 10000;
                submitHighPriorityTask(()->{
                    getString(pageNum, finalFieName, finalJoint,basicId,newBasicId,tableId,map);
                });
            }
        }
        if (Weight.centre.getValue().equals(weight)){
            for (long i = 1; i <= count; i++) {
                long pageNum = (i - 1) * 10000;
                System.out.println(pageNum);
                submitMediumPriorityTask(()->{
                    getString(pageNum, finalFieName, finalJoint,basicId,newBasicId,tableId,map);
                });
            }
        }
        if (Weight.low.getValue().equals(weight)){
            for (long i = 1; i <= count; i++) {
                long pageNum = (i - 1) * 10000;
                submitLowPriorityTask(()->{
                    getString(pageNum, finalFieName, finalJoint,basicId,newBasicId,tableId,map);
                });
            }
        }
        if (Weight.urgency.getValue().equals(weight)){
            for (long i = 1; i <= count; i++) {
                long pageNum = (i - 1) * 10000;
                submitEmergencyTask(()->{
                    getString(pageNum, finalFieName, finalJoint,basicId,newBasicId,tableId,map);
                });
            }
        }
        return null;
    }

    @NotNull
    private void getString(Long pageNum,String fieName,String joint,Long basicId,Long newBasicId,Long tableId,HashMap<String, String> map ) {

        String sql = " SELECT " + fieName + " FROM " + joint;
        System.out.println(sql);
        Result<List<List<DataValue>>> tableValueResult = datasourceFeign.findTableValue(basicId, sql);
        List<List<DataValue>> tableValue = tableValueResult.getData();
        for (List<DataValue> dataValues : tableValue) {
            for (DataValue dataValue : dataValues) {
                String key = dataValue.getKey();
                String newKey = map.get(key);
                dataValue.setKey(newKey);
                log.info(dataValue);
            }
        }
        Result result = datasourceFeign.addProduct(newBasicId, tableId, tableValue);
    }


}
